@@ -9,12 +9,11 @@ pub fn init_logger() {
 
     let enable_logger = env::var("ENABLE_LOGGER")
         .unwrap_or_else(|_| "false".into()) // Default to false if the variable is not set
-        .to_lowercase() == "true";
+        .to_lowercase()
+        == "true";
 
     if enable_logger {
-        Builder::new()
-            .filter(None, LevelFilter::Debug) 
-            .init();
+        Builder::new().filter(None, LevelFilter::Debug).init();
         debug!("Logger is enabled!");
     } else {
         // Configure a minimal logger to suppress debug logs
