@@ -3,3 +3,5 @@ pub mod hinge_and_release_operations;
 pub mod load_assembler;
 pub mod reactions;
 pub mod results;
+pub mod rigid_graph;
+pub mod support_utils;
