@@ -351,11 +351,7 @@ fn gather_external_nodal_wrenches_for_result_type(
     match result_type {
         ResultType::Loadcase(id) => add_case(*id, 1.0),
         ResultType::Loadcombination(cid) => {
-            if let Some(combo) = fers
-                .load_combinations
-                .iter()
-                .find(|lc| lc.load_combination_id == *cid)
-            {
+            if let Some(combo) = fers.load_combinations.iter().find(|lc| lc.id == *cid) {
                 for (case_id, factor) in &combo.load_cases_factors {
                     add_case(*case_id, *factor);
                 }
