@@ -394,7 +394,7 @@ pub fn compute_member_results_from_displacement(
                     if let Some(lc) = fers
                         .load_combinations
                         .iter()
-                        .find(|c| c.load_combination_id == *load_combination_id)
+                        .find(|c| c.id == *load_combination_id)
                     {
                         for (case_id, factor) in &lc.load_cases_factors {
                             if let Some(load_case) =
