@@ -4,7 +4,10 @@ use crate::functions::geometry::{
 };
 use crate::functions::results::{compute_member_results_from_displacement, extract_displacements};
 use crate::functions::rigid_graph::RigidGraph;
-use crate::functions::support_utils::{add_support_springs_to_operator, constrain_single_dof};
+use crate::functions::support_utils::{
+    add_support_springs_to_operator, constrain_single_dof, detect_zero_energy_dofs,
+};
+use crate::models::settings::analysissettings::RigidStrategy;
 use nalgebra::DMatrix;
 use serde::{Deserialize, Serialize};
 use std::collections::{BTreeMap, HashMap, HashSet};
@@ -50,7 +53,21 @@ pub struct FERS {
     pub shape_paths: Option<Vec<ShapePath>>,
 }
 
-const AXIAL_SLACK_TOLERANCE_DEFAULT: f64 = 1.0e-6;
+// ------------------------------
+// Tunable constants (see notes):
+// ------------------------------
+
+// Convergence (relative)
+const REL_DU_TOL: f64 = 1.0e-6; // ||Δu|| / max(1, ||u||)
+const REL_RES_TOL: f64 = 1.0e-6; // ||r|| / max(1, ||f||)
+
+// Incremental loading for second-order
+const LOAD_STEPS_DEFAULT: usize = 2;
+
+// Boundary-condition handling on slave DOFs
+// (reduced to ease conditioning; we also relax ONE_HOT_TOL_RATIO)
+const BC_PENALTY_FACTOR: f64 = 1.0e3;
+const ONE_HOT_TOL_RATIO: f64 = 1.0e-2;
 
 struct RigidElimination {
     s: DMatrix<f64>,
@@ -117,6 +134,65 @@ impl FERS {
         Ok(k)
     }
 
+    fn final_sign_slack(&self) -> f64 {
+        self.settings.analysis_options.axial_slack
+    }
+
+    // Internal: used while iterating the active set
+    #[inline]
+    fn axial_slack_tolerance(&self) -> f64 {
+        2.0 * self.final_sign_slack()
+    }
+
+    // Internal: hysteresis to reactivate a member after it was turned off
+    #[inline]
+    fn axial_reactivation_buffer(&self) -> f64 {
+        1.1 * self.axial_slack_tolerance()
+    }
+
+    fn finalize_tie_strut_consistency(
+        &self,
+        u_full: &nalgebra::DMatrix<f64>,
+        active_map: &mut std::collections::HashMap<u32, bool>,
+        material_map: &std::collections::HashMap<u32, &Material>,
+        section_map: &std::collections::HashMap<u32, &Section>,
+    ) -> bool {
+        use crate::models::members::enums::MemberType;
+        let mut changed = false;
+        let eps = self.final_sign_slack();
+
+        for ms in &self.member_sets {
+            for m in &ms.members {
+                match m.member_type {
+                    MemberType::Tension | MemberType::Compression => {
+                        // If it's already OFF, skip
+                        if !*active_map.get(&m.id).unwrap_or(&true) {
+                            continue;
+                        }
+                        let n = m.calculate_axial_force_3d(u_full, material_map, section_map);
+                        match m.member_type {
+                            MemberType::Tension => {
+                                if n < -eps {
+                                    active_map.insert(m.id, false);
+                                    changed = true;
+                                }
+                            }
+                            MemberType::Compression => {
+                                if n > eps {
+                                    active_map.insert(m.id, false);
+                                    changed = true;
+                                }
+                            }
+                            _ => {}
+                        }
+                    }
+                    _ => {}
+                }
+            }
+        }
+        changed
+    }
+
     fn build_rigid_elimination_partial_using_hinges(&self) -> Result<RigidElimination, String> {
         use std::collections::{HashMap, HashSet};
 
@@ -127,19 +203,47 @@ impl FERS {
             r: (f64, f64, f64),
         }
 
+        let node_has_elastic_or_support = |node_id: u32| -> bool {
+            if self.nodal_supports.iter().any(|s| s.id == node_id) {
+                return true;
+            }
+            for ms in &self.member_sets {
+                for m in &ms.members {
+                    if (m.start_node.id == node_id || m.end_node.id == node_id)
+                        && !matches!(m.member_type, MemberType::Rigid)
+                    {
+                        return true;
+                    }
+                }
+            }
+            false
+        };
+
+        // Build rigid edges once
         let rigid = RigidGraph::build(&self.member_sets)?;
-        let mut rigid_elems: Vec<RigidInfo> = Vec::new();
-        for e in rigid.edges_sorted_master_first() {
-            rigid_elems.push(RigidInfo {
+        let mut rigid_elems: Vec<RigidInfo> = rigid
+            .edges_sorted_master_first()
+            .into_iter()
+            .map(|e| RigidInfo {
                 a: e.master,
                 b: e.slave,
                 r: e.r,
-            });
+            })
+            .collect();
+
+        // Prefer the anchored end as master
+        for info in &mut rigid_elems {
+            let a_anchored = node_has_elastic_or_support(info.a);
+            let b_anchored = node_has_elastic_or_support(info.b);
+            if !a_anchored && b_anchored {
+                std::mem::swap(&mut info.a, &mut info.b);
+                info.r = (-info.r.0, -info.r.1, -info.r.2);
+            }
         }
 
-        let n_full = compute_num_dofs_from_members(&self.member_sets);
+        let number_of_dofs = compute_num_dofs_from_members(&self.member_sets);
 
-        // eliminate all 6 slave DOFs
+        // Eliminate slave DOFs and zero-energy DOFs (except masters)
         let mut eliminated: HashSet<usize> = HashSet::new();
         for info in &rigid_elems {
             for d in 0..6 {
@@ -147,19 +251,25 @@ impl FERS {
             }
         }
 
-        // map retained full -> reduced
+        let zero_energy = detect_zero_energy_dofs(self);
+        let master_nodes: HashSet<u32> = rigid_elems.iter().map(|ri| ri.a).collect();
+        for idx in zero_energy {
+            let node_id: u32 = (idx / 6) as u32 + 1;
+            if !master_nodes.contains(&node_id) {
+                eliminated.insert(idx);
+            }
+        }
+
+        // Map FULL→RED
         let mut full_to_red: HashMap<usize, usize> = HashMap::new();
         let mut red_to_full: Vec<usize> = Vec::new();
         let mut seen: HashSet<usize> = HashSet::new();
         for set in &self.member_sets {
-            for m in &set.members {
-                for node in [&m.start_node, &m.end_node] {
+            for member in &set.members {
+                for node in [&member.start_node, &member.end_node] {
                     for d in 0..6 {
                         let fi = dof_index(node.id, d);
-                        if eliminated.contains(&fi) {
-                            continue;
-                        }
-                        if seen.insert(fi) {
+                        if !eliminated.contains(&fi) && seen.insert(fi) {
                             full_to_red.insert(fi, red_to_full.len());
                             red_to_full.push(fi);
                         }
@@ -169,13 +279,14 @@ impl FERS {
         }
 
         let n_red = red_to_full.len();
-        let mut s = DMatrix::<f64>::zeros(n_full, n_red);
+        let mut s = DMatrix::<f64>::zeros(number_of_dofs, n_red);
 
+        // Identity for retained DOFs
         for (fi, &col) in &full_to_red {
             s[(*fi, col)] = 1.0;
         }
 
-        // fill slave rows: [u_b;θ_b] = C(r)[u_a;θ_a]
+        // Slave rows: [u_b;θ_b] = C(r)[u_a;θ_a]
         for info in &rigid_elems {
             let c = FERS::rigid_map_c(info.r.0, info.r.1, info.r.2);
             for i in 0..6 {
@@ -312,7 +423,93 @@ impl FERS {
                             None
                         }
                     }
-                    MemberType::Rigid => None,
+                    MemberType::Rigid => match self.settings.analysis_options.rigid_strategy {
+                        RigidStrategy::LinearMpc => {
+                            // log::debug!(
+                            //     "RigidStrategy::LinearMpc → skipping rigid member {} in stiffness assembly \
+                            //      (handled via MPC/elimination).",
+                            //     member.id
+                            // );
+                            None
+                        }
+                        RigidStrategy::RigidMember => {
+                            // log::debug!(
+                            //     "RigidStrategy::RigidMember → embedding rigid member {} with boosted E.",
+                            //     member.id
+                            // );
+                            // ------------- pick a base section id -------------
+                            let pick_base_section =
+                                |m: &crate::models::members::member::Member| -> Option<u32> {
+                                    // 1) explicit
+                                    if let Some(id) = m.section {
+                                        return Some(id);
+                                    }
+                                    // 2) neighbor section
+                                    for ms2 in &self.member_sets {
+                                        for mn in &ms2.members {
+                                            if !matches!(mn.member_type, MemberType::Rigid)
+                                                && (mn.start_node.id == m.start_node.id
+                                                    || mn.start_node.id == m.end_node.id
+                                                    || mn.end_node.id == m.start_node.id
+                                                    || mn.end_node.id == m.end_node.id)
+                                            {
+                                                if let Some(sec) = mn.section {
+                                                    return Some(sec);
+                                                }
+                                            }
+                                        }
+                                    }
+                                    // 3) first section in the model
+                                    self.sections.first().map(|s| s.id)
+                                };
+
+                            let base_sec_id = pick_base_section(member).ok_or_else(|| {
+                                format!(
+                                    "RigidMember needs at least one section in the model or a section on rigid member {}.",
+                                    member.id
+                                )
+                            })?;
+
+                            // ------------- get E of that base section -------------
+                            let e_member = self
+                                .sections
+                                .iter()
+                                .find(|s| s.id == base_sec_id)
+                                .and_then(|sec| {
+                                    self.materials.iter().find(|mat| mat.id == sec.material)
+                                })
+                                .map(|mat| mat.e_mod)
+                                .unwrap_or(210.0e9);
+
+                            let e_target = self.max_e_mod_in_model() * 1.0e6;
+                            // guard against absurd factors
+                            let factor = (e_target / e_member.max(1.0)).clamp(1.0, 1.0e8);
+
+                            // ------------- build local K using the base section -------------
+                            let mut tmp: crate::models::members::member::Member = (*member).clone();
+                            tmp.section = Some(base_sec_id);
+
+                            let k_local_base = tmp
+                                .calculate_stiffness_matrix_3d(
+                                    &assembly_context.material_by_id,
+                                    &assembly_context.section_by_id,
+                                )
+                                .ok_or_else(|| {
+                                    format!("Rigid member {} failed to build local K.", member.id)
+                                })?;
+
+                            let k_local_scaled = k_local_base * factor;
+
+                            // no geometric stiff for rigid members (keeps them “kinematic” in 2nd order)
+                            let k_global = local_to_global_with_releases(
+                                &tmp, // use tmp (has the section) for releases/T
+                                &k_local_scaled,
+                                &assembly_context.hinge_by_id,
+                            )?;
+
+                            Some(k_global)
+                        }
+                    },
                 };
 
                 if let Some(element_global) = element_global_opt {
@@ -332,6 +529,20 @@ impl FERS {
         Ok(global_stiffness_matrix)
     }
 
+    fn max_e_mod_in_model(&self) -> f64 {
+        let mut e_max = 0.0_f64;
+        for m in &self.materials {
+            if m.e_mod.is_finite() && m.e_mod > e_max {
+                e_max = m.e_mod;
+            }
+        }
+        if e_max <= 0.0 {
+            210.0e9_f64
+        } else {
+            e_max
+        }
+    }
+
     fn assemble_geometric_stiffness_matrix_with_active(
         &self,
         displacement: &nalgebra::DMatrix<f64>,
@@ -415,37 +626,72 @@ impl FERS {
         use crate::models::members::enums::MemberType;
 
         let mut changed = false;
+        let mut flips: Vec<(u32, &'static str, bool, bool, f64)> = Vec::new();
+        let axial_reactivation_buffer = self.axial_reactivation_buffer();
         for member_set in &self.member_sets {
             for member in &member_set.members {
                 match member.member_type {
-                    MemberType::Tension => {
+                    MemberType::Tension | MemberType::Compression => {
                         let n = member.calculate_axial_force_3d(
                             displacement,
                             material_map,
                             section_map,
                         );
-                        let should_be_active = n >= -axial_slack_tolerance;
-                        if active_map.get(&member.id).copied().unwrap_or(true) != should_be_active {
-                            active_map.insert(member.id, should_be_active);
-                            changed = true;
-                        }
-                    }
-                    MemberType::Compression => {
-                        let n = member.calculate_axial_force_3d(
-                            displacement,
-                            material_map,
-                            section_map,
-                        );
-                        let should_be_active = n <= axial_slack_tolerance;
-                        if active_map.get(&member.id).copied().unwrap_or(true) != should_be_active {
-                            active_map.insert(member.id, should_be_active);
+                        let was = active_map.get(&member.id).copied().unwrap_or(true);
+                        let now = match member.member_type {
+                            MemberType::Tension => {
+                                if was {
+                                    n >= -axial_slack_tolerance
+                                } else {
+                                    n >= axial_reactivation_buffer
+                                }
+                            }
+                            MemberType::Compression => {
+                                if was {
+                                    n <= axial_slack_tolerance
+                                } else {
+                                    n <= -axial_reactivation_buffer
+                                }
+                            }
+                            _ => unreachable!(),
+                        };
+
+                        if was != now {
+                            active_map.insert(member.id, now);
                             changed = true;
+                            let kind = match member.member_type {
+                                MemberType::Tension => "TIE",
+                                MemberType::Compression => "STRUT",
+                                _ => "NA",
+                            };
+                            flips.push((member.id, kind, was, now, n));
                         }
                     }
                     _ => {}
                 }
             }
         }
+
+        if !flips.is_empty() {
+            for (_id, _kind, _was, _now, _n) in &flips {
+                // log::debug!(
+                //     "Active-set flip: member {} ({}) {:>8} → {:>8}  (N = {:.6})",
+                //     id,
+                //     kind,
+                //     if *was { "ACTIVE" } else { "OFF" },
+                //     if *now { "ACTIVE" } else { "OFF" },
+                //     n
+                // );
+            }
+            let _n_on = active_map.values().filter(|v| **v).count();
+            let _n_total = active_map.len();
+            // log::debug!(
+            //     "Active-set summary: {}/{} ties/struts ACTIVE",
+            //     n_on,
+            //     n_total
+            // );
+        }
+
         changed
     }
 
@@ -456,11 +702,11 @@ impl FERS {
         let num_dofs = compute_num_dofs_from_members(&self.member_sets);
         let mut f_comb = DMatrix::<f64>::zeros(num_dofs, 1);
 
-        // Find the combination by its load_combination_id field
+        // Find the combination by its id field
         let combo = self
             .load_combinations
             .iter()
-            .find(|lc| lc.load_combination_id == combination_id)
+            .find(|lc| lc.id == combination_id)
             .ok_or_else(|| format!("LoadCombination {} not found.", combination_id))?;
 
         // Now iterate the HashMap<u32, f64>
@@ -605,10 +851,6 @@ impl FERS {
         let support_map: HashMap<u32, &NodalSupport> =
             self.nodal_supports.iter().map(|s| (s.id, s)).collect();
 
-        // Tunables: penalty factor and one-hot tolerance
-        const BC_PENALTY_FACTOR: f64 = 1.0e8; // increase if constraints look too soft; decrease if conditioning suffers
-        const ONE_HOT_TOL_RATIO: f64 = 1.0e-6; // if all other coefficients are <= 1e-6 of the pivot, treat as one-hot
-
         let n_red: usize = k_red.nrows();
         debug_assert_eq!(n_red, k_red.ncols(), "Reduced stiffness must be square.");
         debug_assert_eq!(
@@ -753,9 +995,9 @@ impl FERS {
         name: String,
         result_type: ResultType,
     ) -> Result<Results, String> {
-        let tolerance: f64 = self.settings.analysis_option.tolerance;
-        let max_it: usize = self.settings.analysis_option.max_iterations.unwrap_or(20) as usize;
-        let axial_slack_tolerance: f64 = AXIAL_SLACK_TOLERANCE_DEFAULT;
+        let tolerance: f64 = self.settings.analysis_options.tolerance;
+        let max_it: usize = self.settings.analysis_options.max_iterations.unwrap_or(20) as usize;
+        let axial_slack_tolerance: f64 = self.axial_slack_tolerance();
 
         let mut active_map = self.init_active_map_tie_comp();
         let mut u_full =
@@ -797,6 +1039,14 @@ impl FERS {
                 &assembly_context.section_by_id,
             );
 
+            // Relative convergence on displacement increment
+            let u_norm = u_full.norm().max(1.0);
+            if (delta.norm() / u_norm) < REL_DU_TOL && !changed {
+                converged = true;
+                break;
+            }
+
+            // Keep older absolute tolerance as a fallback if user set it tighter
             if delta.norm() < tolerance && !changed {
                 converged = true;
                 break;
@@ -810,21 +1060,34 @@ impl FERS {
             ));
         }
 
+        // --- STRICT FINALIZATION (no compression in ties / no tension in struts) ---
+        if self.finalize_tie_strut_consistency(
+            &u_full,
+            &mut active_map,
+            &assembly_context.material_by_id,
+            &assembly_context.section_by_id,
+        ) {
+            // Re-solve once with frozen active_map (no active-set updates here)
+            let k_full = self.build_operator_with_supports(&active_map, None)?;
+            let (mut k_red, mut f_red) = Self::reduce_system(&k_full, &load_vector_full, &elim);
+            self.apply_boundary_conditions_reduced(&elim, &mut k_red, &mut f_red)?;
+            let u_red = k_red.lu().solve(&f_red).ok_or_else(|| {
+                "Reduced stiffness matrix is singular or near-singular in finalization.".to_string()
+            })?;
+            u_full = Self::expand_solution(&elim, &u_red);
+        }
+
         // ---------------------------
         // Build reactions including MPCs
         // ---------------------------
-        // Recompute the FINAL linear operator with the final active_map,
-        // then form the reduced residual r_r = K_r u_r - f_r with the UNMODIFIED K_r, f_r
-        // Map to FULL space: r_full = S r_red
-        // Keep only true support reactions
         let r_support = compose_support_reaction_vector_equilibrium(
             self,
-            &result_type,      // <- pass the ResultType you're solving
-            &u_full,           // <- final full displacement vector
-            Some(&active_map), // <- so ties/struts respect active set
+            &result_type,      // result type you're solving
+            &u_full,           // final full displacement vector
+            Some(&active_map), // ties/struts respect active set
         )?;
 
-        // Store masked reactions
+        // Store masked reactions (optional debug)
         let mut sum_rx = 0.0;
         let mut sum_ry = 0.0;
         let mut sum_rz = 0.0;
@@ -837,7 +1100,6 @@ impl FERS {
             }
         }
 
-        // Sum of EXTERNAL loads per axis (global vector you assembled)
         let mut sum_fx = 0.0;
         let mut sum_fy = 0.0;
         let mut sum_fz = 0.0;
@@ -850,20 +1112,19 @@ impl FERS {
             }
         }
 
-        log::debug!(
-            "Equil check: sum reactions [Fx,Fy,Fz] = [{:.6}, {:.6}, {:.6}]",
-            sum_rx,
-            sum_ry,
-            sum_rz
-        );
-        log::debug!(
-            "Equil check: sum external [Fx,Fy,Fz]  = [{:.6}, {:.6}, {:.6}]",
-            sum_fx,
-            sum_fy,
-            sum_fz
-        );
+        // log::debug!(
+        //     "Equil check: sum reactions [Fx,Fy,Fz] = [{:.6}, {:.6}, {:.6}]",
+        //     sum_rx,
+        //     sum_ry,
+        //     sum_rz
+        // );
+        // log::debug!(
+        //     "Equil check: sum external [Fx,Fy,Fz]  = [{:.6}, {:.6}, {:.6}]",
+        //     sum_fx,
+        //     sum_fy,
+        //     sum_fz
+        // );
 
-        // Store masked reactions
         let results = self
             .build_and_store_results(
                 name.clone(),
@@ -883,10 +1144,8 @@ impl FERS {
         name: String,
         result_type: ResultType,
         max_iterations: usize,
-        tolerance: f64,
+        _tolerance_unused: f64, // superseded by REL_* tolerances
     ) -> Result<Results, String> {
-        let axial_slack_tolerance: f64 = AXIAL_SLACK_TOLERANCE_DEFAULT;
-
         let mut active_map = self.init_active_map_tie_comp();
         let n_full = compute_num_dofs_from_members(&self.member_sets);
         let mut u_full = nalgebra::DMatrix::<f64>::zeros(n_full, 1);
@@ -895,70 +1154,149 @@ impl FERS {
 
         // Build rigid elimination (S) once; it depends only on topology/hinges
         let elim = self.build_rigid_elimination_partial_using_hinges()?;
+        let axial_slack_tolerance: f64 = self.axial_slack_tolerance();
 
-        let mut converged = false;
-        for _iter in 0..max_iterations {
-            // Tangent operator (includes geometric stiffness) for Newton–Raphson
-            let k_tangent_full = self.build_operator_with_supports(&active_map, Some(&u_full))?;
+        // Proportional load stepping
+        let n_steps = LOAD_STEPS_DEFAULT;
+        for step in 1..=n_steps {
+            let lambda = step as f64 / n_steps as f64;
+            let f_lambda_full = &load_vector_full * lambda;
+            let _f_lambda_red = elim.s.transpose() * &f_lambda_full;
 
-            // Reduce: K_tr = Sᵀ K_tangent S, f_r = Sᵀ f
-            let (k_red_tangent, f_red) =
-                Self::reduce_system(&k_tangent_full, &load_vector_full, &elim);
+            // allow at most one active-set change within this step (prevents chatter)
+            let mut changed_once_this_step = false;
+            let mut converged_step = false;
 
-            // Current reduced displacement u_r = Sᵀ u
-            let mut u_red = elim.s.transpose() * &u_full;
+            for _iter in 0..max_iterations {
+                // Tangent at current state (includes geometric stiff if u_full != 0)
+                let k_tangent_full =
+                    self.build_operator_with_supports(&active_map, Some(&u_full))?;
+                let (mut k_red, mut f_red) =
+                    Self::reduce_system(&k_tangent_full, &f_lambda_full, &elim);
 
-            // Residual in reduced space (before BC application): r_r = K_tr u_r − f_r
-            let mut r_red = &k_red_tangent * &u_red - &f_red;
+                // Apply BCs to the reduced system
+                self.apply_boundary_conditions_reduced(&elim, &mut k_red, &mut f_red)?;
 
-            // Apply BCs in REDUCED space, then solve for Δu_r from K_tr,BC Δu_r = −r_r,BC
-            let mut k_treated = k_red_tangent.clone();
-            self.apply_boundary_conditions_reduced(&elim, &mut k_treated, &mut r_red)?;
-            let delta_red = k_treated
-                .lu()
-                .solve(&(-&r_red))
-                .ok_or_else(|| "Tangent stiffness singular.".to_string())?;
+                // Current state in RED and consistent residual
+                let u_red = elim.s.transpose() * &u_full;
+                let r_red = &k_red * &u_red - &f_red;
 
-            // Update reduced, expand to FULL
-            u_red += &delta_red;
-            let u_full_new = Self::expand_solution(&elim, &u_red);
+                // Residual convergence (consistent scaling)
+                let rhs_norm = f_red.norm().max(1.0);
+                let res_now = r_red.norm() / rhs_norm;
+                // log::debug!("  iter: pre-LS res_now = {:.3e}", res_now);
 
-            // Check convergence and update active set
-            let delta_full = &u_full_new - &u_full;
-            u_full = u_full_new;
+                if res_now < REL_RES_TOL {
+                    converged_step = true;
+                    break;
+                }
 
-            let changed = self.update_active_set(
-                &u_full,
-                &mut active_map,
-                axial_slack_tolerance,
-                &assembly_context.material_by_id,
-                &assembly_context.section_by_id,
-            );
+                // Raw Newton correction for treated system
+                let delta_red_raw = k_red
+                    .lu()
+                    .solve(&(-&r_red))
+                    .ok_or_else(|| "Tangent stiffness singular.".to_string())?;
 
-            if delta_full.norm() < tolerance && !changed {
-                converged = true;
-                break;
+                // Backtracking line search around current u_red
+                let mut alpha = 1.0;
+                let c = 1e-4;
+
+                // helper to compute consistent residual at a trial u_red
+                let trial_res = |u_r_trial: &DMatrix<f64>| -> Result<f64, String> {
+                    let u_full_trial = FERS::expand_solution(&elim, u_r_trial);
+                    let k_full2 =
+                        self.build_operator_with_supports(&active_map, Some(&u_full_trial))?;
+                    let (mut k_red2, mut f_red2) =
+                        Self::reduce_system(&k_full2, &f_lambda_full, &elim);
+                    self.apply_boundary_conditions_reduced(&elim, &mut k_red2, &mut f_red2)?;
+                    let r2 = &k_red2 * u_r_trial - &f_red2;
+                    Ok(r2.norm() / f_red2.norm().max(1.0))
+                };
+
+                let mut u_red_trial = &u_red + &(alpha * &delta_red_raw);
+                let mut res_trial = trial_res(&u_red_trial)?;
+
+                while res_trial > (1.0 - c * alpha) * res_now && alpha > 1.0 / 256.0 {
+                    alpha *= 0.5;
+                    u_red_trial = &u_red + &(alpha * &delta_red_raw);
+                    res_trial = trial_res(&u_red_trial)?;
+                }
+                // log::debug!(
+                //     "  iter: line-search α = {:.3e}  res_trial = {:.3e}",
+                //     alpha,
+                //     res_trial
+                // );
+
+                // Accept the step
+                let delta_red = &u_red_trial - &u_red;
+                let du_rel = delta_red.norm() / u_red.norm().max(1.0);
+                // log::debug!(
+                //     "  iter: accepted  ||Δ||/||u|| = {:.3e}  res_trial = {:.3e}",
+                //     du_rel,
+                //     res_trial
+                // );
+
+                // Update u_full
+                u_full = FERS::expand_solution(&elim, &u_red_trial);
+
+                // Active-set update (allow at most once per step)
+                if !changed_once_this_step {
+                    let changed = self.update_active_set(
+                        &u_full,
+                        &mut active_map,
+                        axial_slack_tolerance,
+                        &assembly_context.material_by_id,
+                        &assembly_context.section_by_id,
+                    );
+                    if changed {
+                        changed_once_this_step = true;
+                        // log::debug!(
+                        //     "  iter: ACTIVE-SET CHANGED → restart iteration with new tangent"
+                        // );
+                        continue; // restart the Newton iteration with the new tangent
+                    }
+                }
+
+                // <-- new permissive stop rule -->
+                if res_trial < REL_RES_TOL || du_rel < 1.0e-4 {
+                    converged_step = true;
+                    break;
+                }
             }
-        }
 
-        if !converged {
-            return Err(format!(
-                "Newton–Raphson with active set did not converge in {} iterations",
-                max_iterations
-            ));
+            if !converged_step {
+                return Err(format!(
+                    "Second-order step {}/{} did not converge within {} iterations",
+                    step, n_steps, max_iterations
+                ));
+            }
         }
 
         // ---------------------------
         // Build reactions including MPCs
         // ---------------------------
-        // Use the LINEAR operator (no geometric stiffness) to compute final reactions,
-        // matching classic practice and your previous intent/comments.
-        // Map to FULL space reactions
+        // Use the LINEAR operator (no geometric stiffness) to compute final reactions.
+        if self.finalize_tie_strut_consistency(
+            &u_full,
+            &mut active_map,
+            &assembly_context.material_by_id,
+            &assembly_context.section_by_id,
+        ) {
+            // Re-solve once with frozen active_map (no active-set updates here)
+            let k_full = self.build_operator_with_supports(&active_map, None)?;
+            let (mut k_red, mut f_red) = Self::reduce_system(&k_full, &load_vector_full, &elim);
+            self.apply_boundary_conditions_reduced(&elim, &mut k_red, &mut f_red)?;
+            let u_red = k_red.lu().solve(&f_red).ok_or_else(|| {
+                "Reduced stiffness matrix is singular or near-singular in finalization.".to_string()
+            })?;
+            u_full = Self::expand_solution(&elim, &u_red);
+        }
+
         let r_support = compose_support_reaction_vector_equilibrium(
             self,
-            &result_type,      // <- pass the ResultType you're solving
-            &u_full,           // <- final full displacement vector
-            Some(&active_map), // <- so ties/struts respect active set
+            &result_type,
+            &u_full,
+            Some(&active_map),
         )?;
 
         let results = self
@@ -986,6 +1324,7 @@ impl FERS {
             ResultType::Loadcase(load_case_id),
         )
     }
+
     pub fn solve_for_load_case_second_order(
         &mut self,
         load_case_id: u32,
@@ -1006,12 +1345,13 @@ impl FERS {
             tolerance,
         )
     }
+
     pub fn solve_for_load_combination(&mut self, combination_id: u32) -> Result<Results, String> {
         let load_vector = self.assemble_load_vector_for_combination(combination_id)?;
         let combo = self
             .load_combinations
             .iter()
-            .find(|lc| lc.load_combination_id == combination_id)
+            .find(|lc| lc.id == combination_id)
             .ok_or_else(|| format!("LoadCombination {} not found.", combination_id))?;
         self.solve_first_order_common(
             load_vector,
@@ -1019,6 +1359,7 @@ impl FERS {
             ResultType::Loadcombination(combination_id),
         )
     }
+
     pub fn solve_for_load_combination_second_order(
         &mut self,
         combination_id: u32,
@@ -1029,7 +1370,7 @@ impl FERS {
         let combo = self
             .load_combinations
             .iter()
-            .find(|lc| lc.load_combination_id == combination_id)
+            .find(|lc| lc.id == combination_id)
             .ok_or_else(|| format!("LoadCombination {} not found.", combination_id))?;
         self.solve_second_order_common(
             load_vector,
@@ -1059,7 +1400,7 @@ impl FERS {
         // 2) Node displacements
         let displacement_nodes = extract_displacements(self, displacement_vector);
 
-        // 3) Reactions: already computed via compose_support_reaction_vector_equilibrium(...)
+        // 3) Reactions
         let reaction_nodes: BTreeMap<u32, crate::models::results::reaction::ReactionNodeResult> =
             extract_reaction_nodes(self, global_reaction_vector);
 
