@@ -13,10 +13,9 @@ pub enum LimitState {
 #[derive(Debug, Serialize, Deserialize, ToSchema)]
 pub struct LoadCombination {
     pub id: u32,
-    pub load_combination_id: u32,
     pub name: String,
     pub load_cases_factors: HashMap<u32, f64>,
     pub situation: Option<String>,
     pub check: String,
-    pub limit_state: LimitState,
+    pub limit_state: Option<LimitState>,
 }
