@@ -1,11 +1,9 @@
 // src/models/members/memberset.rs
+use crate::models::members::member::Member;
 use serde::{Deserialize, Serialize};
 use utoipa::ToSchema;
-use crate::models::members::member::Member;
 
-
-#[derive(Serialize, Deserialize, ToSchema
-, Debug)]
+#[derive(Serialize, Deserialize, ToSchema, Debug)]
 pub struct MemberSet {
     pub id: u32,
     pub l_y: Option<f64>,
@@ -13,8 +11,3 @@ pub struct MemberSet {
     pub classification: Option<String>,
     pub members: Vec<Member>,
 }
-
-
-
-
-
