@@ -1,8 +1,7 @@
 use serde::{Deserialize, Serialize};
 use utoipa::ToSchema;
 
-use crate::models::results::forces::{NodeForces};
-
+use crate::models::results::forces::NodeForces;
 
 #[derive(Serialize, Deserialize, ToSchema, Debug, Clone)]
 pub struct MemberResult {
@@ -10,4 +9,4 @@ pub struct MemberResult {
     pub end_node_forces: NodeForces,
     pub maximums: NodeForces,
     pub minimums: NodeForces,
-}
\ No newline at end of file
+}
