@@ -1,7 +1,7 @@
 // src/models/results/reaction.rs
+use crate::models::results::forces::NodeForces;
 use serde::{Deserialize, Serialize};
 use utoipa::ToSchema;
-use crate::models::{results::forces::NodeForces};
 
 #[allow(non_snake_case)]
 #[derive(Serialize, Deserialize, ToSchema, Debug, Clone)]
@@ -11,10 +11,9 @@ pub struct NodeLocation {
     pub Z: f64,
 }
 
-
 #[derive(Serialize, Deserialize, ToSchema, Debug, Clone)]
 pub struct ReactionNodeResult {
     pub nodal_forces: NodeForces,
     pub location: NodeLocation,
-    pub support_id: u32, 
+    pub support_id: u32,
 }
