@@ -1,10 +1,10 @@
-use serde::{Deserialize, Serialize};
-use std::collections::{BTreeMap};
-use utoipa::ToSchema;
 use crate::models::results::memberresult::MemberResult;
 use crate::models::results::reaction::ReactionNodeResult;
 use crate::models::results::resultssummary::ResultsSummary;
 use crate::models::unitychecks::unitycheck::UnityCheck;
+use serde::{Deserialize, Serialize};
+use std::collections::BTreeMap;
+use utoipa::ToSchema;
 
 use super::displacement::NodeDisplacement;
 
@@ -14,9 +14,6 @@ pub enum ResultType {
     Loadcombination(u32),
 }
 
-
-
-
 #[derive(Serialize, Deserialize, ToSchema, Debug, Clone)]
 pub struct Results {
     pub name: String,
@@ -27,5 +24,3 @@ pub struct Results {
     pub summary: ResultsSummary,
     pub unity_checks: Option<BTreeMap<String, UnityCheck>>,
 }
-
-
