@@ -1,4 +1,4 @@
-// only compiled when you do `--features python`
+// Only compiled when you do `--features python`
 use crate::analysis::{
     calculate_from_file_internal, calculate_from_json_internal,
     load_fers_from_file as load_fers_from_file_internal,
@@ -9,22 +9,19 @@ use pyo3::wrap_pyfunction;
 
 #[pyfunction]
 pub fn calculate_from_json(json_data: &str) -> PyResult<String> {
-    // map internal error -> Python RuntimeError
-    calculate_from_json_internal(json_data).map_err(|e| PyRuntimeError::new_err(e.to_string()))
+    calculate_from_json_internal(json_data).map_err(PyRuntimeError::new_err)
 }
 
 #[pyfunction]
 pub fn calculate_from_file(path: &str) -> PyResult<String> {
-    // map internal error -> Python RuntimeError
-    calculate_from_file_internal(path).map_err(|e| PyRuntimeError::new_err(e.to_string()))
+    calculate_from_file_internal(path).map_err(PyRuntimeError::new_err)
 }
 
 #[pyfunction]
 pub fn load_fers_from_file(path: &str) -> PyResult<String> {
-    let fers =
-        load_fers_from_file_internal(path).map_err(|e| PyRuntimeError::new_err(e.to_string()))?;
-
-    serde_json::to_string(&fers).map_err(|e| PyRuntimeError::new_err(e.to_string()))
+    let fers = load_fers_from_file_internal(path)
+        .map_err(|error| PyRuntimeError::new_err(error.to_string()))?;
+    serde_json::to_string(&fers).map_err(|error| PyRuntimeError::new_err(error.to_string()))
 }
 
 #[pymodule]
