@@ -2,14 +2,16 @@
 mod test_support;
 
 use fers_calculations::models::members::memberhinge::MemberHinge;
+use fers_calculations::models::settings::analysissettings::RigidStrategy;
 use fers_calculations::models::supports::supportconditiontype::SupportConditionType;
 use test_support::formulas::*;
 use test_support::helpers::*;
 use test_support::*;
 
-#[test]
-fn test_041_rigid_member_end_load() {
-    let mut model = make_fers();
+// ===================== 041 =====================
+
+fn test_041_rigid_member_end_load_impl(strategy: RigidStrategy) {
+    let mut model = make_fers_with_strategy(strategy);
 
     let mat_id = add_material_s235(&mut model, 1);
     let sec_id = add_section_ipe180_like(&mut model, 1, mat_id, SECOND_MOMENT_STRONG_AXIS_IN_M4);
@@ -70,12 +72,119 @@ fn test_041_rigid_member_end_load() {
     assert_close(dy_3, dy_2 + rz_2 * r_x, TOL_ABSOLUTE_DISPLACEMENT_IN_METER);
 }
 
+#[test]
+fn test_041_rigid_member_end_load_rigid_member() {
+    test_041_rigid_member_end_load_impl(RigidStrategy::RigidMember);
+}
+
+#[test]
+fn test_041_rigid_member_end_load_mpc_linear() {
+    test_041_rigid_member_end_load_impl(RigidStrategy::LinearMpc);
+}
+
+// ===================== 042 =====================
+
+fn test_042_rigid_member_reversed_end_load_impl(strategy: RigidStrategy) {
+    // ---------------- Model setup ----------------
+    let mut model = make_fers_with_strategy(strategy);
+
+    // Material and section
+    let material_id = add_material_s235(&mut model, 1);
+    let section_id =
+        add_section_ipe180_like(&mut model, 1, material_id, SECOND_MOMENT_STRONG_AXIS_IN_M4);
+
+    // Fixed support at node 1
+    model.nodal_supports.push(make_fixed_support(1));
+
+    // Geometry
+    let length_elastic = 5.0_f64;
+    let length_rigid = 5.0_f64;
+    let total_length = length_elastic + length_rigid;
+
+    let node1 = make_node(1, 0.0, 0.0, 0.0, Some(1));
+    let node2 = make_node(2, length_elastic, 0.0, 0.0, None);
+    let node3 = make_node(3, total_length, 0.0, 0.0, None);
+
+    // Members:
+    // - Elastic beam: node1 -> node2
+    // - Rigid link reversed: node3 -> node2
+    let beam_member = make_beam_member(1, &node1, &node2, section_id);
+    let rigid_member_reversed = make_rigid_member(2, &node3, &node2);
+    add_member_set(&mut model, 1, vec![beam_member, rigid_member_reversed]);
+
+    // Load case: 1 kN downward at node 2
+    let load_case_id = add_load_case(&mut model, 1, "End Load");
+    let force_newton = 1000.0_f64;
+    add_nodal_load(
+        &mut model,
+        1,
+        load_case_id,
+        2,
+        force_newton,
+        (0.0, -1.0, 0.0),
+    );
+
+    // ---------------- Solve ----------------
+    model
+        .solve_for_load_case(load_case_id)
+        .expect("Analysis failed");
+
+    let results = model
+        .results
+        .as_ref()
+        .unwrap()
+        .loadcases
+        .get("End Load")
+        .unwrap();
+
+    // Displacements, rotations, and reactions
+    let dy_2 = results.displacement_nodes.get(&2).unwrap().dy;
+    let dy_3 = results.displacement_nodes.get(&3).unwrap().dy;
+    let rz_2 = results.displacement_nodes.get(&2).unwrap().rz;
+    let rz_3 = results.displacement_nodes.get(&3).unwrap().rz;
+    let mz_1 = results.reaction_nodes.get(&1).unwrap().nodal_forces.mz;
+
+    // ---------------- References (analytical) ----------------
+    let e_modulus = 210.0e9_f64;
+    let i_zz = SECOND_MOMENT_STRONG_AXIS_IN_M4;
+
+    let dy_expected = cantilever_end_point_load_deflection_at_free_end(
+        force_newton,
+        length_elastic,
+        e_modulus,
+        i_zz,
+    );
+    let mz_expected =
+        cantilever_end_point_load_fixed_end_moment_magnitude(force_newton, length_elastic);
+    let rz_expected = -force_newton * length_elastic.powi(2) / (2.0 * e_modulus * i_zz);
+
+    let dy_end_from_mid = dy_2 + length_rigid * rz_2;
+
+    // ---------------- Assertions ----------------
+    assert_close(dy_2, dy_expected, TOL_ABSOLUTE_DISPLACEMENT_IN_METER);
+    assert_close(mz_1.abs(), mz_expected, TOL_ABSOLUTE_MOMENT_IN_NEWTON_METER);
+    assert!((rz_2 - rz_3).abs() < TOL_ABSOLUTE_ROTATION_IN_RADIAN);
+    assert!((rz_2 - rz_expected).abs() < TOL_ABSOLUTE_ROTATION_IN_RADIAN);
+    assert_close(dy_3, dy_end_from_mid, TOL_ABSOLUTE_DISPLACEMENT_IN_METER);
+}
+
+#[test]
+fn test_042_rigid_member_reversed_end_load_rigid_member() {
+    test_042_rigid_member_reversed_end_load_impl(RigidStrategy::RigidMember);
+}
+
+#[test]
+fn test_042_rigid_member_reversed_end_load_mpc_linear() {
+    test_042_rigid_member_reversed_end_load_impl(RigidStrategy::LinearMpc);
+}
+
+// ===================== 051 =====================
+
 /// Cantilever with a root rotational spring via a MemberHinge at the start of the elastic member,
 /// preceded by a rigid link: node1 --[RIGID]--> node2 --[NORMAL + spring at start]--> node3.
-#[test]
-fn test_051_member_hinge_root_rotational_spring() {
+fn test_051_member_hinge_root_rotational_spring_impl(strategy: RigidStrategy) {
     // ---------------- Model setup (first order) ----------------
-    let mut model = make_fers();
+    let mut model = make_fers_with_strategy(strategy);
 
     let material_id = add_material_s235(&mut model, 1);
     let section_id =
@@ -168,6 +277,7 @@ fn test_051_member_hinge_root_rotational_spring() {
         + phi_root_expected * length_elastic);
     let reaction_mz_expected = force_newton * (length_elastic + length_rigid);
     let fy_1 = results.reaction_nodes.get(&1).unwrap().nodal_forces.fy;
+
     // ---------------- Assertions (first order) ----------------
     assert_close(
         dy_3,
@@ -183,7 +293,6 @@ fn test_051_member_hinge_root_rotational_spring() {
         TOL_ABSOLUTE_MOMENT_IN_NEWTON_METER,
     );
 
-    // tip rotation and deflection of elastic span with root rotational spring
     assert_close(rz_3, phi_tip_expected, TOL_ABSOLUTE_ROTATION_IN_RADIAN);
     assert_close(
         dy_3,
@@ -191,7 +300,6 @@ fn test_051_member_hinge_root_rotational_spring() {
         TOL_ABSOLUTE_DISPLACEMENT_IN_METER,
     );
 
-    // global equilibrium at the support
     assert_close(fy_1, force_newton, TOL_ABSOLUTE_FORCE_IN_NEWTON);
     assert_close(
         mz_1.abs(),
@@ -200,7 +308,7 @@ fn test_051_member_hinge_root_rotational_spring() {
     );
 
     // ---------------- Second-order sanity (no singular tangent) ----------------
-    let mut model_so = make_fers();
+    let mut model_so = make_fers_with_strategy(strategy);
 
     let material_id_so = add_material_s235(&mut model_so, 1);
     let section_id_so = add_section_ipe180_like(
@@ -277,8 +385,19 @@ fn test_051_member_hinge_root_rotational_spring() {
 }
 
 #[test]
-fn test_061_two_colinear_tension_only_members_with_mid_load() {
-    let mut model = make_fers();
+fn test_051_member_hinge_root_rotational_spring_rigid_member() {
+    test_051_member_hinge_root_rotational_spring_impl(RigidStrategy::RigidMember);
+}
+
+#[test]
+fn test_051_member_hinge_root_rotational_spring_mpc_linear() {
+    test_051_member_hinge_root_rotational_spring_impl(RigidStrategy::LinearMpc);
+}
+
+// ===================== 061 =====================
+
+fn test_061_two_colinear_tension_only_members_with_mid_load_impl(strategy: RigidStrategy) {
+    let mut model = make_fers_with_strategy(strategy);
 
     let mat_id = add_material_s235(&mut model, 1);
     let sec_id = add_section_ipe180_like(&mut model, 1, mat_id, SECOND_MOMENT_STRONG_AXIS_IN_M4);
@@ -332,19 +451,31 @@ fn test_061_two_colinear_tension_only_members_with_mid_load() {
 }
 
 #[test]
-fn test_082_two_base_supports_horizontal_tip_load_reaction_signs_and_equilibrium() {
+fn test_061_two_colinear_tension_only_members_with_mid_load_rigid_member() {
+    test_061_two_colinear_tension_only_members_with_mid_load_impl(RigidStrategy::RigidMember);
+}
+
+#[test]
+fn test_061_two_colinear_tension_only_members_with_mid_load_mpc_linear() {
+    test_061_two_colinear_tension_only_members_with_mid_load_impl(RigidStrategy::LinearMpc);
+}
+
+// ===================== 082 =====================
+
+fn test_082_two_base_supports_horizontal_tip_load_reaction_signs_and_equilibrium_impl(
+    strategy: RigidStrategy,
+) {
     // ------------- Model setup -------------
-    let mut model = make_fers();
+    let mut model = make_fers_with_strategy(strategy);
 
     let material_id = add_material_s235(&mut model, 1);
     let section_id =
         add_section_ipe180_like(&mut model, 1, material_id, SECOND_MOMENT_STRONG_AXIS_IN_M4);
 
-    // Support 1: fully fixed (for nodes 1 and 2, like your JSON setup where both ends of the
-    // horizontal member had the same fixed support in translations and free in rotations).
+    // Support 1: fully fixed
     model.nodal_supports.push(make_fixed_support(1));
 
-    // Support 2: X free, Y free, Z fixed; rotations all free — matches your "id 2" support.
+    // Support 2: X free, Y free, Z fixed; rotations all free
     model.nodal_supports.push(make_support_custom(
         2,
         SupportConditionType::Free,  // Ux
@@ -411,8 +542,7 @@ fn test_082_two_base_supports_horizontal_tip_load_reaction_signs_and_equilibrium
     // No parasitic Z reactions expected
     assert_close(reaction_fz_sum, 0.0, TOL_ABSOLUTE_FORCE_IN_NEWTON);
 
-    // Opposite vertical reactions at the two bases (frame action):
-    // We do not prescribe magnitudes, but they must be opposite in sign and sum to ~0
+    // Opposite vertical reactions at the two bases (frame action)
     let fy1 = reactions_node1.fy;
     let fy2 = reactions_node2.fy;
     assert!(
@@ -426,6 +556,18 @@ fn test_082_two_base_supports_horizontal_tip_load_reaction_signs_and_equilibrium
     // Node 3 has only Uz fixed; it should not pick up significant X or Y reaction.
     assert!(reactions_node3.fx.abs() < TOL_ABSOLUTE_FORCE_IN_NEWTON);
     assert!(reactions_node3.fy.abs() < TOL_ABSOLUTE_FORCE_IN_NEWTON);
+}
 
-    // Optional: sanity on moments could be added here if you want to check ΣM about the origin.
+#[test]
+fn test_082_two_base_supports_horizontal_tip_load_reaction_signs_and_equilibrium_rigid_member() {
+    test_082_two_base_supports_horizontal_tip_load_reaction_signs_and_equilibrium_impl(
+        RigidStrategy::RigidMember,
+    );
+}
+
+#[test]
+fn test_082_two_base_supports_horizontal_tip_load_reaction_signs_and_equilibrium_mpc_linear() {
+    test_082_two_base_supports_horizontal_tip_load_reaction_signs_and_equilibrium_impl(
+        RigidStrategy::LinearMpc,
+    );
 }
